@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __LASSO_DEBUG_H__
@@ -37,6 +36,7 @@ LASSO_EXPORT extern gboolean lasso_flag_memory_debug;
 LASSO_EXPORT extern gboolean lasso_flag_strict_checking;
 LASSO_EXPORT extern gboolean lasso_flag_add_signature;
 LASSO_EXPORT extern gboolean lasso_flag_sign_messages;
+LASSO_EXPORT extern gboolean lasso_flag_thin_sessions;
 
 #ifdef __cplusplus
 }
