@@ -14,7 +14,7 @@ if len(sys.argv) == 2+enable_wsf:
 else:
     srcdir = '.'
 
-regex = re.compile('LASSO_EXPORT.*(lasso_[a-zA-Z0-9_]+).*\(')
+regex = re.compile('LASSO_EXPORT[^;(]*(lasso_[a-zA-Z0-9_]+)', re.MULTILINE)
 
 symbols = []
 for header_file in glob.glob('%s/*/*.h' % srcdir) + glob.glob('%s/*.h' % srcdir) + \
@@ -25,7 +25,7 @@ for header_file in glob.glob('%s/*/*.h' % srcdir) + glob.glob('%s/*.h' % srcdir)
 
 wsf = ['lasso_disco_', 'lasso_dst_', 'lasso_is_', 'lasso_profile_service_',
         'lasso_discovery', 'lasso_wsf', 'lasso_interaction_', 'lasso_utility_',
-        'lasso_sa_', 'lasso_soap_', 'lasso_authentication', 'lasso_wsse_',
+        'lasso_sa_', 'lasso_soap_binding', 'lasso_authentication', 'lasso_wsse_',
         'lasso_sec_', 'lasso_idwsf2', 'lasso_wsf2', 'lasso_wsa_',
         'lasso_wsu_']
 if enable_wsf:
