@@ -18,17 +18,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __LASSO_XML_SOAP11_H__
 #define __LASSO_XML_SOAP11_H__
 
-#include "./soap_body.h"
-#include "./soap_detail.h"
-#include "./soap_envelope.h"
-#include "./soap_fault.h"
-#include "./soap_header.h"
+#include "soap_body.h"
+#include "soap_detail.h"
+#include "soap_envelope.h"
+#include "soap_fault.h"
+#include "soap_header.h"
 
 #endif /* __LASSO_XML_SOAP11_H__ */
