@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -39,7 +38,7 @@
  *
  */
 
-#include "./wsse_username_token.h"
+#include "wsse_username_token.h"
 #include "../idwsf_strings.h"
 #include <xmlsec/xmltree.h>
 #include <openssl/sha.h>
