@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -33,7 +32,7 @@
 #include "../lasso/backward_comp.h"
 #include "../lasso/xml/saml-2.0/samlp2_logout_request.h"
 #include "../bindings/ghashtable.h"
-#include "./tests.h"
+#include "tests.h"
 
 static char*
 generateIdentityProviderContextDump()
@@ -93,7 +92,7 @@ generateIdentityProviderContextDumpMemory()
 	char *metadata = NULL;
 	char *private_key = NULL;
 	char *certificate = NULL;
-	guint len;
+	size_t len;
 	char *ret = NULL;
 
 	g_file_get_contents(TESTSDATADIR "/idp5-saml2/metadata.xml", &metadata, &len, NULL);
@@ -682,22 +681,352 @@ START_TEST(test04_sso_then_slo_soap)
 }
 END_TEST
 
+START_TEST(test05_sso_idp_with_key_rollover)
+{
+	LassoServer *idpContext1 = NULL;
+	LassoServer *idpContext2 = NULL;
+	LassoServer *spContext = NULL;
+	LassoLogin *idpLoginContext1 = NULL;
+	LassoLogin *idpLoginContext2 = NULL;
+	LassoLogin *spLoginContext = NULL;
+
+	/* Create an IdP context for IdP initiated SSO with private key 1 */
+	idpContext1 = lasso_server_new(
+			TESTSDATADIR "idp11-multikey-saml2/metadata.xml",
+			TESTSDATADIR "idp11-multikey-saml2/private-key-1.pem",
+			NULL, /* Secret key to unlock private key */
+			TESTSDATADIR "idp11-multikey-saml2/certificate-1.pem");
+	check_not_null(idpContext1)
+	check_good_rc(lasso_server_add_provider(
+			idpContext1,
+			LASSO_PROVIDER_ROLE_SP,
+			TESTSDATADIR "/sp6-saml2/metadata.xml",
+			NULL,
+			NULL));
+	/* Create an IdP context for IdP initiated SSO with private key 2 */
+	idpContext2 = lasso_server_new(
+			TESTSDATADIR "idp11-multikey-saml2/metadata.xml",
+			TESTSDATADIR "idp11-multikey-saml2/private-key-2.pem",
+			NULL, /* Secret key to unlock private key */
+			TESTSDATADIR "idp11-multikey-saml2/certificate-2.pem");
+	check_not_null(idpContext2)
+	check_good_rc(lasso_server_add_provider(
+			idpContext2,
+			LASSO_PROVIDER_ROLE_SP,
+			TESTSDATADIR "/sp6-saml2/metadata.xml",
+			NULL,
+			NULL));
+	/* Create an SP context */
+	spContext = lasso_server_new(
+			TESTSDATADIR "/sp6-saml2/metadata.xml",
+			TESTSDATADIR "/sp6-saml2/private-key.pem",
+			NULL, /* Secret key to unlock private key */
+			NULL);
+	check_not_null(spContext)
+	check_good_rc(lasso_server_add_provider(
+			spContext,
+			LASSO_PROVIDER_ROLE_IDP,
+			TESTSDATADIR "/idp11-multikey-saml2/metadata.xml",
+			NULL,
+			NULL));
+
+	/* Create login contexts */
+	idpLoginContext1 = lasso_login_new(idpContext1);
+	check_not_null(idpLoginContext1);
+	idpLoginContext2 = lasso_login_new(idpContext2);
+	check_not_null(idpLoginContext2);
+	spLoginContext = lasso_login_new(spContext);
+	check_not_null(spLoginContext);
+
+	/* Create first response signed with key 1*/
+	check_good_rc(lasso_login_init_idp_initiated_authn_request(idpLoginContext1, "http://sp6/metadata"));
+	lasso_assign_string(LASSO_SAMLP2_AUTHN_REQUEST(idpLoginContext1->parent.request)->ProtocolBinding,
+			LASSO_SAML2_METADATA_BINDING_POST);
+	check_good_rc(lasso_login_process_authn_request_msg(idpLoginContext1, NULL));
+	check_good_rc(lasso_login_validate_request_msg(idpLoginContext1,
+			1, /* authentication_result */
+		        0 /* is_consent_obtained */
+			));
+
+	check_good_rc(lasso_login_build_assertion(idpLoginContext1,
+			LASSO_SAML_AUTHENTICATION_METHOD_PASSWORD,
+			"FIXME: authenticationInstant",
+			"FIXME: reauthenticateOnOrAfter",
+			"FIXME: notBefore",
+			"FIXME: notOnOrAfter"));
+	check_good_rc(lasso_login_build_authn_response_msg(idpLoginContext1));
+	check_not_null(idpLoginContext1->parent.msg_body);
+	check_not_null(idpLoginContext1->parent.msg_url);
+
+	/* Create second response signed with key 2 */
+	check_good_rc(lasso_login_init_idp_initiated_authn_request(idpLoginContext2, "http://sp6/metadata"));
+	lasso_assign_string(LASSO_SAMLP2_AUTHN_REQUEST(idpLoginContext2->parent.request)->ProtocolBinding,
+			LASSO_SAML2_METADATA_BINDING_POST);
+	check_good_rc(lasso_login_process_authn_request_msg(idpLoginContext2, NULL));
+	check_good_rc(lasso_login_validate_request_msg(idpLoginContext2,
+			1, /* authentication_result */
+		        0 /* is_consent_obtained */
+			));
+
+	check_good_rc(lasso_login_build_assertion(idpLoginContext2,
+			LASSO_SAML_AUTHENTICATION_METHOD_PASSWORD,
+			"FIXME: authenticationInstant",
+			"FIXME: reauthenticateOnOrAfter",
+			"FIXME: notBefore",
+			"FIXME: notOnOrAfter"));
+	check_good_rc(lasso_login_build_authn_response_msg(idpLoginContext2));
+	check_not_null(idpLoginContext2->parent.msg_body);
+	check_not_null(idpLoginContext2->parent.msg_url);
+
+	/* Process response 1 */
+	check_good_rc(lasso_login_process_authn_response_msg(spLoginContext,
+				idpLoginContext1->parent.msg_body));
+	check_good_rc(lasso_login_accept_sso(spLoginContext));
+
+	/* Process response 2 */
+	block_lasso_logs;
+	check_good_rc(lasso_login_process_authn_response_msg(spLoginContext,
+				idpLoginContext2->parent.msg_body));
+	unblock_lasso_logs;
+	check_good_rc(lasso_login_accept_sso(spLoginContext));
+
+	/* Cleanup */
+	lasso_release_gobject(idpLoginContext1);
+	lasso_release_gobject(idpLoginContext2);
+	lasso_release_gobject(spLoginContext);
+	lasso_release_gobject(idpContext1);
+	lasso_release_gobject(idpContext2);
+	lasso_release_gobject(spContext);
+}
+END_TEST
+
+#define make_context(ctx, server_prefix, server_suffix, provider_role, \
+		provider_prefix, provider_suffix) \
+	ctx =  lasso_server_new( \
+			TESTSDATADIR server_prefix "/metadata" server_suffix ".xml", \
+			TESTSDATADIR server_prefix "/private-key" server_suffix ".pem", \
+			NULL, /* Secret key to unlock private key */ \
+			TESTSDATADIR server_prefix "/certificate" server_suffix ".pem"); \
+	check_not_null(ctx); \
+	check_good_rc(lasso_server_add_provider( \
+			ctx, \
+			provider_role, \
+			TESTSDATADIR provider_prefix "/metadata" provider_suffix ".xml", \
+			NULL, \
+			NULL)); \
+	providers = g_hash_table_get_values(ctx->providers); \
+	check_not_null(providers); \
+	lasso_provider_set_encryption_mode(LASSO_PROVIDER(providers->data), \
+			LASSO_ENCRYPTION_MODE_ASSERTION | LASSO_ENCRYPTION_MODE_NAMEID); \
+	g_list_free(providers);
+
+void
+sso_sp_with_key_rollover(LassoServer *idp_context, LassoServer *sp_context)
+{
+	LassoLogin *idp_login_context;
+	LassoLogin *sp_login_context;
+
+	check_not_null(idp_login_context = lasso_login_new(idp_context));
+	check_not_null(sp_login_context = lasso_login_new(sp_context))
+
+	/* Create response */
+	check_good_rc(lasso_login_init_idp_initiated_authn_request(idp_login_context,
+				"http://sp11/metadata"));
+
+	lasso_assign_string(LASSO_SAMLP2_AUTHN_REQUEST(idp_login_context->parent.request)->ProtocolBinding,
+			LASSO_SAML2_METADATA_BINDING_POST);
+	lasso_assign_string(LASSO_SAMLP2_AUTHN_REQUEST(idp_login_context->parent.request)->NameIDPolicy->Format,
+			LASSO_SAML2_NAME_IDENTIFIER_FORMAT_PERSISTENT);
+	LASSO_SAMLP2_AUTHN_REQUEST(idp_login_context->parent.request)->NameIDPolicy->AllowCreate = 1;
+
+	block_lasso_logs;
+	check_good_rc(lasso_login_process_authn_request_msg(idp_login_context, NULL));
+	unblock_lasso_logs;
+	check_good_rc(lasso_login_validate_request_msg(idp_login_context,
+			1, /* authentication_result */
+		        0 /* is_consent_obtained */
+			));
+
+	check_good_rc(lasso_login_build_assertion(idp_login_context,
+			LASSO_SAML_AUTHENTICATION_METHOD_PASSWORD,
+			"FIXME: authenticationInstant",
+			"FIXME: reauthenticateOnOrAfter",
+			"FIXME: notBefore",
+			"FIXME: notOnOrAfter"));
+	check_good_rc(lasso_login_build_authn_response_msg(idp_login_context));
+	check_not_null(idp_login_context->parent.msg_body);
+	check_not_null(idp_login_context->parent.msg_url);
+
+	/* Process response */
+	block_lasso_logs;
+	check_good_rc(lasso_login_process_authn_response_msg(sp_login_context,
+				idp_login_context->parent.msg_body));
+	unblock_lasso_logs;
+	check_good_rc(lasso_login_accept_sso(sp_login_context));
+
+	/* Cleanup */
+	lasso_release_gobject(idp_login_context);
+	lasso_release_gobject(sp_login_context);
+}
+
+START_TEST(test06_sso_sp_with_key_rollover)
+{
+	LassoServer *idp_context_before_rollover = NULL;
+	LassoServer *idp_context_after_rollover = NULL;
+	LassoServer *sp_context_before_rollover = NULL;
+	LassoServer *sp_context_after_rollover = NULL;
+	GList *providers;
+
+	/* Create an IdP context for IdP initiated SSO with provider metadata 1 */
+	make_context(idp_context_before_rollover, "idp6-saml2", "", LASSO_PROVIDER_ROLE_SP,
+			"sp11-multikey-saml2", "-before-rollover")
+	make_context(idp_context_after_rollover, "idp6-saml2", "", LASSO_PROVIDER_ROLE_SP,
+			"sp11-multikey-saml2", "-after-rollover")
+	make_context(sp_context_before_rollover, "sp11-multikey-saml2", "-before-rollover",
+			LASSO_PROVIDER_ROLE_IDP, "idp6-saml2", "")
+	lasso_server_set_encryption_private_key(sp_context_before_rollover,
+			TESTSDATADIR "sp11-multikey-saml2/private-key-after-rollover.pem");
+	make_context(sp_context_after_rollover, "sp11-multikey-saml2", "-after-rollover",
+			LASSO_PROVIDER_ROLE_IDP, "idp6-saml2", "")
+	lasso_server_set_encryption_private_key(sp_context_after_rollover,
+			TESTSDATADIR "sp11-multikey-saml2/private-key-before-rollover.pem");
+
+	/* Tests... */
+	sso_sp_with_key_rollover(idp_context_before_rollover, sp_context_before_rollover);
+	sso_sp_with_key_rollover(idp_context_after_rollover, sp_context_before_rollover);
+	sso_sp_with_key_rollover(idp_context_before_rollover, sp_context_after_rollover);
+	sso_sp_with_key_rollover(idp_context_after_rollover, sp_context_after_rollover);
+
+	/* Cleanup */
+	lasso_release_gobject(idp_context_before_rollover);
+	lasso_release_gobject(idp_context_after_rollover);
+	lasso_release_gobject(sp_context_before_rollover);
+	lasso_release_gobject(sp_context_after_rollover);
+}
+END_TEST
+
+#define test07_make_context(ctx, server_prefix, provider_role, provider_prefix, key) \
+	ctx =  lasso_server_new( \
+			TESTSDATADIR server_prefix "/metadata.xml", \
+			NULL, \
+			NULL, /* Secret key to unlock private key */ \
+			NULL); \
+	check_not_null(ctx); \
+	check_good_rc(lasso_server_add_provider( \
+			ctx, \
+			provider_role, \
+			TESTSDATADIR provider_prefix "/metadata.xml", \
+			NULL, \
+			NULL)); \
+	providers = g_hash_table_get_values(ctx->providers); \
+	check_not_null(providers); \
+	lasso_provider_set_server_signing_key(LASSO_PROVIDER(providers->data), \
+			key); \
+	lasso_provider_add_key(LASSO_PROVIDER(providers->data), key, FALSE); \
+	g_list_free(providers);
+
+static void
+sso_initiated_by_sp(LassoServer *idp_context, LassoServer *sp_context)
+{
+	LassoLogin *idp_login_context;
+	LassoLogin *sp_login_context;
+	char *authn_request_query;
+
+	check_not_null(idp_login_context = lasso_login_new(idp_context));
+	check_not_null(sp_login_context = lasso_login_new(sp_context))
+
+	/* Create response */
+	check_good_rc(lasso_login_init_authn_request(sp_login_context, NULL, LASSO_HTTP_METHOD_REDIRECT));
+
+	lasso_assign_string(LASSO_SAMLP2_AUTHN_REQUEST(sp_login_context->parent.request)->ProtocolBinding,
+			LASSO_SAML2_METADATA_BINDING_POST);
+	lasso_assign_string(LASSO_SAMLP2_AUTHN_REQUEST(sp_login_context->parent.request)->NameIDPolicy->Format,
+			LASSO_SAML2_NAME_IDENTIFIER_FORMAT_PERSISTENT);
+	LASSO_SAMLP2_AUTHN_REQUEST(sp_login_context->parent.request)->NameIDPolicy->AllowCreate = 1;
+	check_good_rc(lasso_login_build_authn_request_msg(sp_login_context));
+	check_not_null(sp_login_context->parent.msg_url);
+	authn_request_query = strchr(sp_login_context->parent.msg_url, '?');
+	check_not_null(authn_request_query);
+	authn_request_query += 1;
+	check_good_rc(lasso_login_process_authn_request_msg(idp_login_context, authn_request_query));
+
+	check_good_rc(lasso_login_validate_request_msg(idp_login_context,
+			1, /* authentication_result */
+		        0 /* is_consent_obtained */
+			));
+
+	check_good_rc(lasso_login_build_assertion(idp_login_context,
+			LASSO_SAML_AUTHENTICATION_METHOD_PASSWORD,
+			"FIXME: authenticationInstant",
+			"FIXME: reauthenticateOnOrAfter",
+			"FIXME: notBefore",
+			"FIXME: notOnOrAfter"));
+	check_good_rc(lasso_login_build_authn_response_msg(idp_login_context));
+	check_not_null(idp_login_context->parent.msg_body);
+	check_not_null(idp_login_context->parent.msg_url);
+
+	/* Process response */
+	check_good_rc(lasso_login_process_authn_response_msg(sp_login_context,
+				idp_login_context->parent.msg_body));
+	check_good_rc(lasso_login_accept_sso(sp_login_context));
+
+	/* Cleanup */
+	lasso_release_gobject(idp_login_context);
+	lasso_release_gobject(sp_login_context);
+}
+
+START_TEST(test07_sso_sp_with_hmac_sha1_signatures)
+{
+	LassoServer *idp_context = NULL;
+	LassoServer *sp_context = NULL;
+	GList *providers;
+	LassoKey *key = NULL;
+
+	/* Create the shared key */
+	key = lasso_key_new_for_signature_from_memory("xxxxxxxxxxxxxxxx", 16,
+			NULL, LASSO_SIGNATURE_METHOD_HMAC_SHA1, NULL);
+	check_true(LASSO_IS_KEY(key));
+
+	/* Create an IdP context for IdP initiated SSO with provider metadata 1 */
+	test07_make_context(idp_context, "idp6-saml2", LASSO_PROVIDER_ROLE_SP, "sp6-saml2", key)
+	test07_make_context(sp_context, "sp6-saml2", LASSO_PROVIDER_ROLE_IDP, "idp6-saml2", key)
+
+	block_lasso_logs;
+	sso_initiated_by_sp(idp_context, sp_context);
+	unblock_lasso_logs;
+
+	/* Cleanup */
+	lasso_release_gobject(idp_context);
+	lasso_release_gobject(sp_context);
+	lasso_release_gobject(key);
+}
+END_TEST
+
 Suite*
 login_saml2_suite()
 {
-	Suite *s = suite_create("Login");
+	Suite *s = suite_create("Login using SAML 2.0");
 	TCase *tc_generate = tcase_create("Generate Server Contexts");
 	TCase *tc_spLogin = tcase_create("Login initiated by service provider");
 	TCase *tc_spLoginMemory = tcase_create("Login initiated by service provider without key loading");
 	TCase *tc_spSloSoap = tcase_create("Login initiated by service provider without key loading and with SLO SOAP");
+	TCase *tc_idpKeyRollover = tcase_create("Login initiated by idp, idp use two differents signing keys (simulate key roll-over)");
+	TCase *tc_spKeyRollover = tcase_create("Login initiated by idp, sp use two differents encrypting keys (simulate key roll-over)");
+	TCase *tc_hmacSignature = tcase_create("Login initiated by sp, using shared-key signature");
 	suite_add_tcase(s, tc_generate);
 	suite_add_tcase(s, tc_spLogin);
 	suite_add_tcase(s, tc_spLoginMemory);
 	suite_add_tcase(s, tc_spSloSoap);
+	suite_add_tcase(s, tc_idpKeyRollover);
+	suite_add_tcase(s, tc_spKeyRollover);
+	suite_add_tcase(s, tc_hmacSignature);
 	tcase_add_test(tc_generate, test01_saml2_generateServersContextDumps);
 	tcase_add_test(tc_spLogin, test02_saml2_serviceProviderLogin);
 	tcase_add_test(tc_spLoginMemory, test03_saml2_serviceProviderLogin);
 	tcase_add_test(tc_spSloSoap, test04_sso_then_slo_soap);
+	tcase_add_test(tc_idpKeyRollover, test05_sso_idp_with_key_rollover);
+	tcase_add_test(tc_spKeyRollover, test06_sso_sp_with_key_rollover);
+	tcase_add_test(tc_hmacSignature, test07_sso_sp_with_hmac_sha1_signatures);
 	return s;
 }
 
